<?php

App::uses('EvTwitterAppController', 'EvTwitter.Controller');
App::uses('Twitter', 'EvTwitter.Utility');

class TwitterConnectController extends EvTwitterAppController {

	public $uses = 'EvTwitter.TwitterConnection';

	public function beforeFilter() {

		$this->adminActions[] = 'admin_twitter';
		$this->adminActions[] = 'admin_clear';

		return parent::beforeFilter();
	}

/**
 * Manages a default admin index view of this model
 * Includes a filter form and paginated results
 *
 *  @return void
 */
	public function admin_index() {

		if ($this->TwitterConnection->isTwitterSetup()) {

			$this->Session->setFlash('Your website is already connected to twitter', 'flash_success');

		} else {

			$Twitter = new Twitter();

			$temporaryCredentials = $Twitter->getRequestToken(Router::url(array('action' => 'twitter'), true));

			$this->Session->write('EvTwitter.tmp', $temporaryCredentials);

			$this->redirect($Twitter->getAuthorizeURL($temporaryCredentials));

			return;

		}

		return $this->redirect(array('controller' => 'site_settings', 'action' => 'index', 'plugin' => false));
	}

/**
 * Callback function for twitter once it has connected a user to the app
 * Saves the users OAuth details and redirects them back to the admin_index
 */
	public function admin_twitter() {

		if (isset($this->params->query['denied'])) {
			$this->Session->setFlash('Failed to connect app to twitter', 'flash_fail');
			return $this->redirect(array('controller' => 'site_settings', 'action' => 'index', 'plugin' => false));
		}

		$temporaryCredentials = $this->Session->read('EvTwitter.tmp');

		$Twitter = new Twitter(
			$temporaryCredentials['oauth_token'],
			$temporaryCredentials['oauth_token_secret']
		);

		$tokenCredentials = $Twitter->getAccessToken($_REQUEST['oauth_verifier']);

		$data = array(
			'token' => $tokenCredentials['oauth_token'],
			'secret' => $tokenCredentials['oauth_token_secret'],
		);

		$this->TwitterConnection->create();
		$this->TwitterConnection->save($data);

		$this->Session->setFlash('You have successfully connected your website to twitter', 'flash_success');
		return $this->redirect(array('controller' => 'site_settings', 'action' => 'index', 'plugin' => false));

	}

/**
 * Function to disassociate a site with Twitter
 * Deleting TwitterOauth record and all tweets
 * @param $id - Site ID
 */
	public function admin_clear($id) {

		//Get twitter Oauth row
		$oauth = $this->TwitterConnection->find('first');
		if ($oauth) {
			$this->TwitterConnection->delete($oauth['TwitterConnection']['id']);
			$this->Session->setFlash('You have successfully unlinked your website from twitter', 'flash_success');
			return $this->redirect(array('controller'=>'site_settings', 'action'=>'index', 'plugin' => false));
		} else {
			$this->Session->setFlash('Your account is not currently linked to twitter. ', 'flash_fail');
			return $this->redirect(array('controller'=>'site_settings', 'action'=>'index', 'plugin' => false));
		}
	}

}