<?php

App::uses('TwitterTweet', 'EvTwitter.Model');
App::uses('EvTwitterAppModel', 'EvTwitter.Model');

/**
 * TwitterOauth Model
 * Author: Sam Biggins
 *
 * @property Site $Site
 */
class TwitterConnection extends EvTwitterAppModel {

	/**
	 * Display field
	 *
	 * @var string
	 */
	public $displayField = 'oauth_token';


	/**
	 *  Runs before delete
	 *  - Deletes all tweets
	 */
	public function beforeDelete($cascade = true) {
		//Delete tweets
		$this->TwitterTweet = new TwitterTweet();
		$this->TwitterTweet->deleteAll();

		return true;
	}

/**
 * Method to check whether a site has set up thier OAUTH twitter details
 * @return (boolean) - Returns true if the user id is set
 */
	public function isTwitterSetup() {

		return $this->find('first') != false;
		
	}

	/**
	 * Method to return the most recent tweets for the given site
	 * Uses a Twitter Application which users give permission to.
	 * Returns false if the site has not got a twitter account associated with it.
	 */
	public function getSiteTweets($limit = 5) {
		App::uses('Twitter', 'Twitter.Utility');


		$oauth_tokens = $this->find('first');

		if(!$oauth_tokens)
			return false;

		//We cache the tweets so check when the last time we retrieved tweets was.
		// Need to retrieve new tweets every hour
		// $last_checked = new DateTime($oauth_tokens['TwitterOauth']['last_checked']);

		$hours = 1; //Check every 1 hours

		if(strtotime($oauth_tokens['TwitterOauth']['last_checked']) + ($hours * 60 * 60) < strtotime('now'))
		{
			//Fetch new tweets
			$this->fetchNewTweets($oauth_tokens);
		}

		//Fetch tweets from cache
		$this->Tweet = new TwitterTweet();
		$tweets = $this->Tweet->find('all', array(
				'limit' => $limit,
				'order' => 'created DESC'
			)
		);

		return $tweets;


	}

	/**
	 * Retreives users tweets from Twitter using API and App permission
	 */
	private function fetchNewTweets($oauth_tokens = null)
	{
		$consumerKey = Configure::read('Twitter.consumerKey');
		$consumerSecret = Configure::read('Twitter.consumerSecret');

		if($oauth_tokens === null)
		{
			$oauth_tokens = $this->find('first');
		}

		$this->_Twitter = new Twitter();
		$this->_Twitter->setupApp($consumerKey, $consumerSecret);

		$this->_Twitter->setToken($oauth_tokens['TwitterOauth']['oauth_token'], $oauth_tokens['TwitterOauth']['oauth_secret']);

		$this->Tweet = new TwitterTweet();

		$last_checked = 0;
		if (! is_null($oauth_tokens['TwitterOauth']['last_checked'])) {
			$last_checked = strtotime($oauth_tokens['TwitterOauth']['last_checked']);
		}

		// NOTE: Could this be done with just one save?
		$user_timeline = $this->_Twitter->userTimeline();
		if (is_array($user_timeline) && ! empty($user_timeline)) {

			foreach($user_timeline as $tweet)
			{
				$tweet_time = strtotime($tweet['created_at']);

				if (($tweet_time > $last_checked) || is_null($oauth_tokens['TwitterOauth']['last_checked'])) {

					$this->Tweet->create();

					$data = array(
						'twitter_id' => $tweet['id_str'],
						'tweet' => $tweet['text'],
						'created' => date("Y-m-d H:i:s", $tweet_time),
					);

					$this->Tweet->set($data);
					//Will only save tweet if twitter_id is unique
					$this->Tweet->save();
				}
			}
		}

		//Update the time twitter was last checked
		$this->read(null, $oauth_tokens[$this->alias]['id']);
		$this->set('last_checked', date("Y-m-d H:i:s") );
		$this->save();
	}
}
